@@ -1,18 +1,40 @@
 from email.parser import HeaderParser, BytesHeaderParser
 from io import BytesIO, IOBase
-from typing import List, Optional, Union, Dict, Mapping, Iterator, Tuple, Iterable
+from typing import (
+    List,
+    Optional,
+    Union,
+    Dict,
+    Mapping,
+    Iterator,
+    Tuple,
+    Iterable,
+    Any,
+    NoReturn,
+)
 from email.header import decode_header
-from cached_property import cached_property
-from requests import Response
-from requests.structures import CaseInsensitiveDict
+from re import findall, IGNORECASE, escape
+from kiss_headers.structures import CaseInsensitiveDict
 from copy import deepcopy
 
 RESERVED_KEYWORD = [
-    'and_', 'assert_', 'in_', 'not_',
-    'pass_', 'finally_', 'while_',
-    'yield_', 'is_', 'as_', 'break_',
-    'return_', 'elif_', 'except_', 'def_',
-    'from_'
+    "and_",
+    "assert_",
+    "in_",
+    "not_",
+    "pass_",
+    "finally_",
+    "while_",
+    "yield_",
+    "is_",
+    "as_",
+    "break_",
+    "return_",
+    "elif_",
+    "except_",
+    "def_",
+    "from_",
+    "for_",
 ]
 
 
@@ -33,23 +55,24 @@ class Header(object):
     samesite: str
     domain: str
 
-    def __init__(self, head: str, content: str):
+    def __init__(self, name: str, content: str):
 
-        self._head: str = head
+        self._name: str = name
+        self._normalized_name: str = Header.normalize_name(self._name)
         self._content: str = content
 
-        self._members: List[str] = [el.lstrip() for el in self._content.split(';')]
+        self._members: List[str] = [el.lstrip() for el in self._content.split(";")]
 
         self._not_valued_attrs: List[str] = list()
         self._valued_attrs: Dict[str, Union[str, List[str]]] = dict()
         self._valued_attrs_normalized: Dict[str, Union[str, List[str]]] = dict()
 
         for member in self._members:
-            if '=' in member:
-                key, value = tuple(member.split('=', maxsplit=1))
+            if "=" in member:
+                key, value = tuple(member.split("=", maxsplit=1))
 
                 # avoid confusing base64 look alike single value for (key, value)
-                if value.count('=') == len(value) or len(value) == 0 or ' ' in key:
+                if value.count("=") == len(value) or len(value) == 0 or " " in key:
                     self._not_valued_attrs.append(member)
                     continue
 
@@ -61,7 +84,9 @@ class Header(object):
                     else:
                         self._valued_attrs[key].append(value)
 
-                self._valued_attrs_normalized[Header.normalize_name(key)] = self._valued_attrs[key]
+                self._valued_attrs_normalized[
+                    Header.normalize_name(key)
+                ] = self._valued_attrs[key]
                 continue
 
             self._not_valued_attrs.append(member)
@@ -71,21 +96,21 @@ class Header(object):
         """
         Normalize header name or attribute name by applying lowercase and replacing '-' to '_'.
         """
-        return name.lower().replace('-', '_')
+        return name.lower().replace("-", "_")
 
     @property
     def name(self) -> str:
         """
         Output the original header name as it was captured initially
         """
-        return self._head
+        return self._name
 
-    @cached_property
+    @property
     def normalized_name(self) -> str:
         """
         Output header name but normalized, lower case and '-' character become '_'.
         """
-        return Header.normalize_name(self.name)
+        return self._normalized_name
 
     @property
     def content(self) -> str:
@@ -98,16 +123,117 @@ class Header(object):
 
         return self._content
 
-    def __deepcopy__(self, memodict: Dict) -> 'Header':
+    def __deepcopy__(self, memodict: Dict) -> "Header":
         return Header(deepcopy(self.name), deepcopy(self.content))
 
+    def __setattr__(self, key: str, value: str) -> NoReturn:
+        """
+        Set attribute on header using the property notation.
+        """
+
+        # Avoid conflict with __init__ sequence of Header
+        if key in [
+            "_name",
+            "_normalized_name",
+            "_content",
+            "_members",
+            "_not_valued_attrs",
+            "_valued_attrs_normalized",
+            "_valued_attrs",
+        ]:
+            return super().__setattr__(key, value)
+
+        if key[0] == "_":
+            key = key[1:]
+
+        if key.lower() in RESERVED_KEYWORD:
+            key = key[:-1]
+
+        self[key] = value
+
+    def __setitem__(self, key: str, value: str) -> NoReturn:
+        """
+        Set an attribute bracket syntax like. This will erase previously set attribute named after the key.
+        """
+        key_normalized = Header.normalize_name(key)
+
+        if key in self:
+            del self[key]
+
+        self._valued_attrs[key] = value
+        self._valued_attrs_normalized[key_normalized] = self._valued_attrs[key]
+
+        self._content += '; {key}="{value}"'.format(key=key, value=value)
+
+    def __delitem__(self, key: str) -> NoReturn:
+        """
+        Remove any attribute named after the key in header using the bracket syntax.
+           >>> del headers.content_type['charset']
+        """
+        key_normalized = Header.normalize_name(key)
+
+        if key_normalized not in self._valued_attrs_normalized:
+            raise KeyError(
+                "'{item}' attribute is not defined within '{header}' header.".format(
+                    item=key, header=self.name
+                )
+            )
+
+        del self._valued_attrs_normalized[key]
+        not_normalized_keys = self._valued_attrs.keys()
+
+        for key_ in not_normalized_keys:
+            if Header.normalize_name(key_) == key_normalized:
+                del self._valued_attrs[key_]
+                break
+
+        for elem in findall(
+            r"{key_name}=.*?(?=[;\n])".format(key_name=escape(key)),
+            self._content + "\n",
+            IGNORECASE,
+        ):
+
+            has_semicolon_at_the_end: bool = False
+
+            try:
+                self._content.index(elem + ";")
+                has_semicolon_at_the_end = True
+            except ValueError:
+                pass
+
+            self._content: str = self._content.replace(
+                elem + (";" if has_semicolon_at_the_end else ""), ""
+            ).rstrip(" ").lstrip(" ")
+
+            if self._content.startswith(";"):
+                self._content = self._content[1:]
+
+            if self._content.endswith(";"):
+                self._content = self._content[:-1]
+
+    def __delattr__(self, item: str) -> NoReturn:
+        """
+        Remove any attribute named after the key in header using the property notation.
+           >>> del headers.content_type.charset
+        """
+        item = Header.normalize_name(item)
+
+        if item not in self._valued_attrs_normalized:
+            raise AttributeError(
+                "'{item}' attribute is not defined within '{header}' header.".format(
+                    item=item, header=self.name
+                )
+            )
+
+        del self[item]
+
     def __iter__(self) -> Iterator[Tuple[str, Optional[str]]]:
         for key, value in self._valued_attrs.items():
             yield key, self[key]
         for adjective in self._not_valued_attrs:
             yield adjective, None
 
-    def __eq__(self, other: Union[str, 'Header']) -> bool:
+    def __eq__(self, other: Union[str, "Header"]) -> bool:
         """
         Verify equality between a Header object and str or another Header object.
         If testing against str, the first thing is to match it to raw content, if not equal verify if not in members.
@@ -115,8 +241,15 @@ class Header(object):
         if isinstance(other, str):
             return self.content == other or other in self._not_valued_attrs
         if isinstance(other, Header):
-            return self.normalized_name == other.normalized_name and self.content == other.content
-        raise TypeError('Cannot compare type {type_} to an Header. Use str or Header.'.format(type_=type(other)))
+            return (
+                self.normalized_name == other.normalized_name
+                and self.content == other.content
+            )
+        raise TypeError(
+            "Cannot compare type {type_} to an Header. Use str or Header.".format(
+                type_=type(other)
+            )
+        )
 
     def __str__(self) -> str:
         """
@@ -128,7 +261,7 @@ class Header(object):
         """
         Unambiguous representation of a single header.
         """
-        return "{head}: {content}".format(head=self._head, content=self._content)
+        return "{head}: {content}".format(head=self._name, content=self._content)
 
     def __dir__(self) -> Iterable[str]:
         """
@@ -172,7 +305,10 @@ class Header(object):
             value = self._valued_attrs_normalized[normalized_item]
         else:
             raise KeyError(
-                "'{item}' attribute is not defined within '{header}' header.".format(item=item, header=self.name))
+                "'{item}' attribute is not defined within '{header}' header.".format(
+                    item=item, header=self.name
+                )
+            )
 
         # Unquote value if necessary
         if isinstance(value, str) and value.startswith('"') and value.endswith('"'):
@@ -185,8 +321,21 @@ class Header(object):
         All the magic happen here, this method should be invoked when trying to call (not declared) properties.
         For instance, calling self.charset should end up here and be replaced by self['charset'].
         """
-        if item not in self._valued_attrs and Header.normalize_name(item) not in self._valued_attrs_normalized:
-            raise AttributeError("'{item}' attribute is not defined within '{header}' header.".format(item=item, header=self.name))
+        if item[0] == "_":
+            item = item[1:]
+
+        if item.lower() in RESERVED_KEYWORD:
+            item = item[:-1]
+
+        if (
+            item not in self._valued_attrs
+            and Header.normalize_name(item) not in self._valued_attrs_normalized
+        ):
+            raise AttributeError(
+                "'{item}' attribute is not defined within '{header}' header.".format(
+                    item=item, header=self.name
+                )
+            )
 
         return self[item]
 
@@ -279,7 +428,7 @@ class Headers:
             return None
         return self[header]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[Header]:
         for header in self._headers:
             yield header
 
@@ -289,6 +438,13 @@ class Headers:
         """
         return list(set([header.name for header in self]))
 
+    def items(self) -> Iterator[Tuple[str, str]]:
+        """
+        Provide an iterator witch each entry contain a tuple of header name and content.
+        """
+        for header in self:
+            yield header.name, header.content
+
     def to_dict(self) -> CaseInsensitiveDict:
         """
         Provide a CaseInsensitiveDict output of current headers. This output type has been borrowed from psf/requests.
@@ -298,18 +454,18 @@ class Headers:
         dict_headers = CaseInsensitiveDict()
 
         for header in self:
-            header_name_no_underscore = header.name.replace('_', '-')
+            header_name_no_underscore = header.name.replace("_", "-")
             if header_name_no_underscore not in dict_headers:
                 dict_headers[header_name_no_underscore] = header.content
                 continue
-            dict_headers[header_name_no_underscore] += ', '+header.content
+            dict_headers[header_name_no_underscore] += ", " + header.content
 
         return dict_headers
 
-    def __deepcopy__(self, memodict: Dict) -> 'Headers':
+    def __deepcopy__(self, memodict: Dict) -> "Headers":
         return Headers(deepcopy(self._headers))
 
-    def __delitem__(self, key: str):
+    def __delitem__(self, key: str) -> NoReturn:
         """
         Remove all matching header named after called key.
            >>> del headers['content-type']
@@ -318,7 +474,9 @@ class Headers:
         to_be_removed = []
 
         if key not in self:
-            raise KeyError("'{item}' header is not defined in headers.".format(item=key))
+            raise KeyError(
+                "'{item}' header is not defined in headers.".format(item=key)
+            )
 
         for header in self:
             if header.normalized_name == key:
@@ -327,7 +485,7 @@ class Headers:
         for header in to_be_removed:
             self._headers.remove(header)
 
-    def __setitem__(self, key: str, value: str):
+    def __setitem__(self, key: str, value: str) -> NoReturn:
         """
         Set header using the bracket syntax. This operation would remove any existing header named after the key.
         """
@@ -336,26 +494,28 @@ class Headers:
 
         self._headers.append(Header(key, value))
 
-    def __delattr__(self, item: str):
+    def __delattr__(self, item: str) -> NoReturn:
         """
         Remove header using the property notation.
            >>> del headers.content_type
         """
         if item not in self:
-            raise AttributeError("'{item}' header is not defined in headers.".format(item=item))
+            raise AttributeError(
+                "'{item}' header is not defined in headers.".format(item=item)
+            )
 
         del self[item]
 
-    def __setattr__(self, key: str, value: str):
+    def __setattr__(self, key: str, value: str) -> NoReturn:
         """
         Set header like it is a property/member. This operation would remove any existing header named after the key.
         """
-        if key == '_headers':
+        if key == "_headers":
             return super().__setattr__(key, value)
 
         self[key] = value
 
-    def __eq__(self, other: 'Headers') -> bool:
+    def __eq__(self, other: "Headers") -> bool:
         """
         Basically compare if one Headers instance equal to another. Order does not matter and instance length matter.
         """
@@ -374,7 +534,7 @@ class Headers:
         """
         return len(self._headers)
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         Just calling __repr__ of self. see __repr__.
         """
@@ -384,9 +544,9 @@ class Headers:
         """
         Non-ambiguous representation of an Headers instance.
         """
-        return '\n'.join([header.__repr__() for header in self])
+        return "\n".join([header.__repr__() for header in self])
 
-    def __add__(self, other: Header) -> 'Headers':
+    def __add__(self, other: Header) -> "Headers":
         """
         Add using syntax c = a + b. The result is a newly created object.
         """
@@ -395,7 +555,7 @@ class Headers:
 
         return headers
 
-    def __sub__(self, other: Union[Header, str]) -> 'Headers':
+    def __sub__(self, other: Union[Header, str]) -> "Headers":
         """
         Subtract using syntax c = a - b. The result is a newly created object.
         """
@@ -404,7 +564,7 @@ class Headers:
 
         return headers
 
-    def __iadd__(self, other: Header) -> 'Headers':
+    def __iadd__(self, other: Header) -> "Headers":
         """
         Inline add, using operator '+'. It is only possible to add to it another Header object.
         """
@@ -412,9 +572,11 @@ class Headers:
             self._headers.append(other)
             return self
 
-        raise TypeError('Cannot add type "{type_}" to Headers.'.format(type_=str(type(other))))
+        raise TypeError(
+            'Cannot add type "{type_}" to Headers.'.format(type_=str(type(other)))
+        )
 
-    def __isub__(self, other: Union[Header, str]) -> 'Headers':
+    def __isub__(self, other: Union[Header, str]) -> "Headers":
         """
         Inline subtract, using operator '-'. If a str is subtracted to it,
         would be looking for header named like provided str.
@@ -440,7 +602,9 @@ class Headers:
                 self._headers.remove(other)
                 return self
 
-        raise TypeError('Cannot subtract type "{type_}" to Headers.'.format(type_=str(type(other))))
+        raise TypeError(
+            'Cannot subtract type "{type_}" to Headers.'.format(type_=str(type(other)))
+        )
 
     def __getitem__(self, item: Union[str, int]) -> Union[Header, List[Header]]:
         """
@@ -449,7 +613,9 @@ class Headers:
         item = Header.normalize_name(item)
 
         if item not in self:
-            raise KeyError("'{item}' header is not defined in headers.".format(item=item))
+            raise KeyError(
+                "'{item}' header is not defined in headers.".format(item=item)
+            )
 
         headers: List[Header] = list()
 
@@ -461,17 +627,21 @@ class Headers:
 
     def __getattr__(self, item: str) -> Union[Header, List[Header]]:
         """
-        Where the magic happen, every header are accessible via the property notation. eg.
+        Where the magic happen, every header are accessible via the property notation.
+        The result is either a single Header or a list of Header.
+        eg.
            >>> headers.content_type
         """
-        if item[0] == '_':
+        if item[0] == "_":
             item = item[1:]
 
         if item.lower() in RESERVED_KEYWORD:
             item = item[:-1]
 
         if item not in self:
-            raise AttributeError("'{item}' header is not defined in headers.".format(item=item))
+            raise AttributeError(
+                "'{item}' header is not defined in headers.".format(item=item)
+            )
 
         return self[item]
 
@@ -495,42 +665,42 @@ class Headers:
         Provide a better auto-completion when using python interpreter. We are feeding __dir__ so Python can be aware
         of what properties are callable. In other word, more precise auto-completion when not using IDE.
         """
-        return super().__dir__() + list(set([header.normalized_name for header in self]))
+        return super().__dir__() + list(
+            set([header.normalized_name for header in self])
+        )
 
 
-def parse_it(raw_headers: Union[bytes, str, Dict[str, str], IOBase, Response]) -> Headers:
+def parse_it(raw_headers: Any) -> Headers:
     """
     Just decode anything that could contain headers. That simple PERIOD.
     """
+
+    headers: Optional[List[Tuple[str, Any]]] = None
+
     if isinstance(raw_headers, str):
         headers = HeaderParser().parsestr(raw_headers, headersonly=True).items()
     elif isinstance(raw_headers, bytes) or isinstance(raw_headers, IOBase):
-        buf = BytesIO(raw_headers) if not hasattr(raw_headers, 'closed') else raw_headers
-        bytes_ = buf.read()
-
-        try:
-            bytes_.decode('ascii')
-        except UnicodeDecodeError:
-            try:
-                bytes_.decode('utf-8')
-            except UnicodeDecodeError:
-                raise UnicodeDecodeError(
-                    'utf-8', bytes_, 0, len(bytes_),
-                    'You intended to parse headers that are neither from ASCII encoding or UTF-8.')
-        
-        buf.seek(0)
+        buf: BytesIO = (
+            BytesIO(raw_headers) if not hasattr(raw_headers, "closed") else raw_headers
+        )
         headers = BytesHeaderParser().parse(buf, headersonly=True).items()
     elif isinstance(raw_headers, Mapping):
         headers = raw_headers.items()
-    elif isinstance(raw_headers, Response):
-        headers = list()
-        for header_name in raw_headers.raw.headers:
-            for header_content in raw_headers.raw.headers.getlist(header_name):
-                headers.append(
-                    (header_name, header_content)
-                )
     else:
-        raise TypeError('Cannot parse type {type_} as it is not supported by kiss-header.'.format(type_=type(raw_headers)))
+        r = findall(r"<class '([a-zA-Z0-9.]+)'>", str(type(raw_headers)))
+
+        if r and r[0] == "requests.models.Response":
+            headers = []
+            for header_name in raw_headers.raw.headers:
+                for header_content in raw_headers.raw.headers.getlist(header_name):
+                    headers.append((header_name, header_content))
+
+    if headers is None:
+        raise TypeError(
+            "Cannot parse type {type_} as it is not supported by kiss-header.".format(
+                type_=type(raw_headers)
+            )
+        )
 
     revised_headers = list()
 
@@ -541,15 +711,21 @@ def parse_it(raw_headers: Union[bytes, str, Dict[str, str], IOBase, Response]) -
             if isinstance(partial, str):
                 revised_content += partial
             if isinstance(partial, bytes):
-                revised_content += partial.decode(partial_encoding if partial_encoding is not None else 'utf-8', errors='ignore')
+                revised_content += partial.decode(
+                    partial_encoding if partial_encoding is not None else "utf-8",
+                    errors="ignore",
+                )
 
         revised_headers.append((head, revised_content))
 
     # Sometime raw content does not begin with headers. If that is the case, search for the next line.
-    if len(revised_headers) == 0 and len(raw_headers) > 0 and (isinstance(raw_headers, bytes) or isinstance(raw_headers, str)):
+    if (
+        len(revised_headers) == 0
+        and len(raw_headers) > 0
+        and (isinstance(raw_headers, bytes) or isinstance(raw_headers, str))
+    ):
         next_iter = raw_headers.split(
-            b'\n' if isinstance(raw_headers, bytes) else '\n',
-            maxsplit=1
+            b"\n" if isinstance(raw_headers, bytes) else "\n", maxsplit=1
         )
 
         if len(next_iter) >= 2:
